/*
 * $Id: hack103.h,v 1.1.1.1 2005/09/18 22:05:45 dhmunro Exp $
 */
/* Copyright (c) 2005, The Regents of the University of California.
 * All rights reserved.
 * This file is part of yorick (http://yorick.sourceforge.net).
 * Read the accompanying LICENSE file for details.
 */

#define sqrt (*sqrt_phack)
#define tanh tanh_hack
#define sinh sinh_hack
#define hypot hypot_hack
