#define P_CHAR_ALIGN 1

#define P_SHORT_SIZE 2L
#define P_SHORT_ALIGN 2
#define P_SHORT_ORDER 1

#define P_INT_SIZE 4L
#define P_INT_ALIGN 4
#define P_INT_ORDER 1

#define P_LONG_SIZE 8L
#define P_LONG_ALIGN 8
#define P_LONG_ORDER 1

#define P_FLOAT_SIZE 4L
#define P_FLOAT_ALIGN 4
#define P_FLOAT_ORDER 1
#define P_FLOAT_LAYOUT 0, 1, 8, 9, 23, 0, 127L

#define P_DOUBLE_SIZE 8L
#define P_DOUBLE_ALIGN 8
#define P_DOUBLE_ORDER 1
#define P_DOUBLE_LAYOUT 0, 1, 11, 12, 52, 0, 1023L

#define P_POINTER_SIZE 8L
#define P_POINTER_ALIGN 8

#define P_STRUCT_ALIGN 1
