//{{NO_DEPENDENCIES}}

#define IDD_ABOUTBOX                    100
#define IDP_OLE_INIT_FAILED             100
#define ID_VIEW_TERM                    101
#define ID_VIEW_HIST                    102
#define IDP_FAILED_TO_CREATE            103
#define IDR_MAINFRAME                   128
#define IDR_EDITFRAME                   129
#define IDD_GOTO_LINE                   130
#define IDC_GOTO_LINE                   1003
#define ID_CANCEL_EDIT_CNTR             32768
#define ID_SIGINT                       32775
#define ID_CALL_FUNC                    32776
#define ID_GOTO_LINE                    32777
#define ID_GOTO_OUT                     32778
#define ID_SELECT_OUT                   32779
#define ID_OPEN_LINE                    32780
#define ID_CHOOSE_FILE                  32781

#ifdef APSTUDIO_INVOKED
#ifndef APSTUDIO_READONLY_SYMBOLS
#define _APS_3D_CONTROLS                     1
#define _APS_NEXT_RESOURCE_VALUE        132
#define _APS_NEXT_COMMAND_VALUE         32782
#define _APS_NEXT_CONTROL_VALUE         1004
#define _APS_NEXT_SYMED_VALUE           103
#endif
#endif
